package org.jboss.qa.brms.drl.compile.example;

import java.lang.management.ManagementFactory;
import java.lang.management.MemoryPoolMXBean;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.io.ResourceFactory;
import org.junit.Test;

public class CreateKBaseFrom10kRulesTest {
    private static int NANOS_PER_SEC = 1000 * 1000 * 1000;
    private static int LOOPS = 5;

    @Test
    public void testCreateKBasefrom10kDrl() throws Exception {
        printPermgenUsageOnStdout();
        for (int i = 1; i <= LOOPS; i++) {
            System.out.println("Running loop " + i);
            createKBase("drl-kbase-creation-10k.drl", ResourceType.DRL);
            //createKBase("dtable1-kbase-creation.xls", ResourceType.DTABLE);
            System.gc();
            printPermgenUsageOnStdout();
        }
        System.gc();
        System.out.println("End of the test, after manually calling GC...");
        printPermgenUsageOnStdout();
    }

    private KnowledgeBase createKBase(String resourceName, ResourceType type) {
        KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
        long measureStart = System.nanoTime();
        kbuilder.add(ResourceFactory.newClassPathResource(resourceName), type);
        if (kbuilder.hasErrors()) {
            System.out.println("Error occured when compling package!");
            throw new RuntimeException(kbuilder.getErrors().toString());
        }

        KnowledgeBase kbase = KnowledgeBaseFactory.newKnowledgeBase();

        kbase.addKnowledgePackages(kbuilder.getKnowledgePackages());
        long measureEnd = System.nanoTime();
        System.out.println("Just to be sure the code is not eliminated: packages=" + kbase.getKnowledgePackages().size());
        System.out.println("KBase building took " + (measureEnd - measureStart) / NANOS_PER_SEC + "s.");
        return kbase;

    }

    private void printPermgenUsageOnStdout() {
        // this is JVM dependent, should work on Oracle JDK and OpenJDK
        for (MemoryPoolMXBean mx : ManagementFactory.getMemoryPoolMXBeans()) {
            if ("PS Perm Gen".equals(mx.getName())) {
                System.out.println(String.format("Permgen usage: %dMiB", mx.getUsage().getUsed() / 1024 / 1024));
            }
        }
    }

}
